#include "radial_vibration.h"
#include "ui_radial_vibration.h"
#include <QFile>
#include <QDebug>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>

#include "data_config.h"
#include "vibrationdata.h"
#include "config_mgr.h"

Radial::Radial(int slot_no_, int channel_, bool active, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Radial_vibration) {
    ui->setupUi(this);
    slot_no = slot_no_;
    channel = channel_;
    QString chan = QString("%1").arg(channel);
    QString slot = QString("%1").arg(slot_no);
    ui->label_channel->setText(chan);
    ui->label_slot->setText(slot);
    if (active) {
        ui->label_active->setText("（启用）");
    } else {
        ui->label_active->setText("（停用）");
    }
    Init();
}

Radial::~Radial() {
    delete ui;
}

void Radial::on_pushButton_cancel_clicked() {
    this->close();
}

void Radial::on_checkBox_1x_ampl_toggled(bool checked) {
    if (checked) {
        ui->comboBox_1x_value_range->setEnabled(true);
        ui->doubleSpinBox_1x_ampl_clamp->setEnabled(true);
        ui->doubleSpinBox_1x_phase_lag_clamp->setEnabled(true);
        return;
    }
    ui->comboBox_1x_value_range->setEnabled(false);
    ui->doubleSpinBox_1x_ampl_clamp->setEnabled(false);
    ui->doubleSpinBox_1x_phase_lag_clamp->setEnabled(false);
}

void Radial::on_checkBox_2x_ampl_toggled(bool checked) {
    if (checked) {
        ui->comboBox_2x_value_range->setEnabled(true);
        ui->doubleSpinBox_2x_ampl_clamp->setEnabled(true);
        ui->doubleSpinBox_2x_phase_lag_clamp->setEnabled(true);
        return;
    }
    ui->comboBox_2x_value_range->setEnabled(false);
    ui->doubleSpinBox_2x_ampl_clamp->setEnabled(false);
    ui->doubleSpinBox_2x_phase_lag_clamp->setEnabled(false);
}

void Radial::Init() {
    std::shared_ptr<CardBase> base_ptr = ConfigMgr::Instance()->GetSlotPtr(slot_no);
    if (base_ptr == nullptr) {
        qCritical() << "[Radial::Init] should not be here";
        return;
    }
    std::shared_ptr<VibrationData> ptr = std::dynamic_pointer_cast<VibrationData>(base_ptr);
    if (ptr->base_config_[channel - 1].channel_type != kVibRadial) {
        qDebug() << "[Radial::Init] channel type changes";
        return;
    }
    std::shared_ptr<VariableBase> variable_base = ptr->GetChannelPtr(channel);
    if (variable_base == nullptr) {
        qDebug() << "[Radial::Init] no channel ptr";
        return;
    }
    std::shared_ptr<RadialVariable> variable_ptr = std::dynamic_pointer_cast<RadialVariable>(variable_base);
    ui->checkBox_low_pass->setChecked(variable_ptr->filter_[0].checked);
    ui->spinBox_low_pass_low->setValue(variable_ptr->filter_[0].low);
    ui->spinBox_low_pass_high->setValue(variable_ptr->filter_[0].high);
    ui->checkBox_high_pass->setChecked(variable_ptr->filter_[1].checked);
    ui->spinBox_high_pass_low->setValue(variable_ptr->filter_[1].low);
    ui->spinBox_high_pass_high->setValue(variable_ptr->filter_[1].high);
    ui->checkBox_band_pass->setChecked(variable_ptr->filter_[2].checked);
    ui->spinBox_band_pass_low->setValue(variable_ptr->filter_[2].low);
    ui->spinBox_band_pass_high->setValue(variable_ptr->filter_[2].high);
    ui->comboBox_direct_value_range->setCurrentIndex(variable_ptr->direct_.full_scale_range);
    ui->doubleSpinBox_direct_clamp->setValue(variable_ptr->direct_.clamp_value);
//    ui->label_bias_voltage->setText(QString::number(variable_ptr->direct_.bias_voltage));
//    ui->doubleSpinBox_bias_volt_clamp->setValue(variables[i].clamp_value);
//    ui->comboBox_bias_volt_range->setCurrentIndex();
    ui->checkBox_1x_ampl->setChecked(variable_ptr->x1_.checked);
    ui->comboBox_1x_value_range->setCurrentIndex(variable_ptr->x1_.full_scale_range);
    ui->doubleSpinBox_1x_ampl_clamp->setValue(variable_ptr->x1_.clamp_value);
    ui->doubleSpinBox_1x_phase_lag_clamp->setValue(variable_ptr->x1_.phase_lag);
    ui->checkBox_2x_ampl->setChecked(variable_ptr->x2_.checked);
    ui->comboBox_2x_value_range->setCurrentIndex(variable_ptr->x2_.full_scale_range);
    ui->doubleSpinBox_2x_ampl_clamp->setValue(variable_ptr->x2_.clamp_value);
    ui->doubleSpinBox_2x_phase_lag_clamp->setValue(variable_ptr->x2_.phase_lag);
    ui->spinBox_alert->setValue(variable_ptr->delay_.alert);
    ui->doubleSpinBox_danger->setValue(variable_ptr->delay_.danger);
    ui->checkBox_100ms->setChecked(variable_ptr->delay_.active_100ms);
    ui->checkBox_alert_latching->setChecked(variable_ptr->alert_latching_);
    ui->checkBox_danger_latching->setChecked(variable_ptr->danger_latching_);
    ui->comboBox_recorder_output->setCurrentIndex(variable_ptr->recorder_out_.recorder_output);
    ui->checkBox_two_ma_clamp->setChecked(variable_ptr->recorder_out_.two_ma_clamp);
    ui->doubleSpinBox_trip_multiply->setValue(variable_ptr->recorder_out_.trip_multiply);
    ui->comboBox_comparision->setCurrentIndex(variable_ptr->recorder_out_.comparision);
    ui->spinBox_comparision_percentage->setValue(variable_ptr->recorder_out_.percentage);
}

void Radial::on_pushButton_confirm_clicked() {
    std::shared_ptr<CardBase> base_ptr = ConfigMgr::Instance()->GetSlotPtr(slot_no);
    if (base_ptr == nullptr) {
        qCritical() << "[Radial::Init] should not be here";
        return;
    }
    std::shared_ptr<VibrationData> ptr = std::dynamic_pointer_cast<VibrationData>(base_ptr);
    std::shared_ptr<VariableBase> variable_base = ptr->GetChannelPtr(channel);
    if (variable_base == nullptr || variable_base->type_ != kVibRadial) {
        if (variable_base == nullptr) {
            qDebug() << "no channel ptr";
        } else {
            ptr->RemoveChannel(channel);
            qDebug() << "channel type change";
        }
        std::shared_ptr<RadialVariable> variable = std::make_shared<RadialVariable>();
        variable->id_ = channel;
        variable->type_ = kVibRadial;
        variable->filter_[0].checked = ui->checkBox_low_pass->isChecked();
        variable->filter_[0].low = ui->spinBox_low_pass_low->value();
        variable->filter_[0].high = ui->spinBox_low_pass_high->value();
        variable->filter_[1].checked = ui->checkBox_high_pass->isChecked();
        variable->filter_[1].low = ui->spinBox_high_pass_low->value();
        variable->filter_[1].high = ui->spinBox_high_pass_high->value();
        variable->filter_[2].checked = ui->checkBox_band_pass->isChecked();
        variable->filter_[2].low = ui->spinBox_band_pass_low->value();
        variable->filter_[2].high = ui->spinBox_band_pass_high->value();
        variable->direct_.full_scale_range = ui->comboBox_direct_value_range->currentIndex();
        variable->direct_.clamp_value = ui->doubleSpinBox_direct_clamp->value();
        variable->direct_.custom = 0; // TODO:
        variable->x1_.checked = ui->checkBox_1x_ampl->isChecked();
        variable->x1_.full_scale_range = ui->comboBox_1x_value_range->currentIndex();
        variable->x1_.clamp_value = ui->doubleSpinBox_1x_ampl_clamp->value();
        variable->x1_.custom = 0; // TODO:
        variable->x1_.phase_lag = ui->doubleSpinBox_1x_phase_lag_clamp->value();
        variable->x2_.checked = ui->checkBox_2x_ampl->isChecked();
        variable->x2_.full_scale_range = ui->comboBox_2x_value_range->currentIndex();
        variable->x2_.clamp_value = ui->doubleSpinBox_2x_ampl_clamp->value();
        variable->x2_.custom = 0; // TODO:
        variable->x2_.phase_lag = ui->doubleSpinBox_2x_phase_lag_clamp->value();
        variable->recorder_out_.recorder_output = ui->comboBox_recorder_output->currentIndex();
        variable->recorder_out_.two_ma_clamp = ui->checkBox_two_ma_clamp->isChecked();
        variable->recorder_out_.trip_multiply = ui->doubleSpinBox_trip_multiply->value();
        variable->recorder_out_.comparision = ui->comboBox_comparision->currentIndex();
        variable->recorder_out_.percentage = ui->spinBox_comparision_percentage->value();
        variable->delay_.alert = ui->spinBox_alert->value();
        variable->delay_.danger = ui->doubleSpinBox_danger->value();
        variable->delay_.active_100ms = ui->checkBox_100ms->isChecked();
        variable->alert_latching_ = ui->checkBox_alert_latching->isChecked();
        variable->danger_latching_ = ui->checkBox_danger_latching->isChecked();
        ptr->variables_.push_back(variable);
        this->close();
        return;
    }
    std::shared_ptr<RadialVariable> variable = std::dynamic_pointer_cast<RadialVariable>(variable_base);
    variable->filter_[0].checked = ui->checkBox_low_pass->isChecked();
    variable->filter_[0].low = ui->spinBox_low_pass_low->value();
    variable->filter_[0].high = ui->spinBox_low_pass_high->value();
    variable->filter_[1].checked = ui->checkBox_high_pass->isChecked();
    variable->filter_[1].low = ui->spinBox_high_pass_low->value();
    variable->filter_[1].high = ui->spinBox_high_pass_high->value();
    variable->filter_[2].checked = ui->checkBox_band_pass->isChecked();
    variable->filter_[2].low = ui->spinBox_band_pass_low->value();
    variable->filter_[2].high = ui->spinBox_band_pass_high->value();
    variable->direct_.full_scale_range = ui->comboBox_direct_value_range->currentIndex();
    variable->direct_.clamp_value = ui->doubleSpinBox_direct_clamp->value();
    variable->direct_.custom = 0; // TODO:
    variable->x1_.checked = ui->checkBox_1x_ampl->isChecked();
    variable->x1_.full_scale_range = ui->comboBox_1x_value_range->currentIndex();
    variable->x1_.clamp_value = ui->doubleSpinBox_1x_ampl_clamp->value();
    variable->x1_.custom = 0; // TODO:
    variable->x1_.phase_lag = ui->doubleSpinBox_1x_phase_lag_clamp->value();
    variable->x2_.checked = ui->checkBox_2x_ampl->isChecked();
    variable->x2_.full_scale_range = ui->comboBox_2x_value_range->currentIndex();
    variable->x2_.clamp_value = ui->doubleSpinBox_2x_ampl_clamp->value();
    variable->x2_.custom = 0; // TODO:
    variable->x2_.phase_lag = ui->doubleSpinBox_2x_phase_lag_clamp->value();
    variable->recorder_out_.recorder_output = ui->comboBox_recorder_output->currentIndex();
    variable->recorder_out_.two_ma_clamp = ui->checkBox_two_ma_clamp->isChecked();
    variable->recorder_out_.trip_multiply = ui->doubleSpinBox_trip_multiply->value();
    variable->recorder_out_.comparision = ui->comboBox_comparision->currentIndex();
    variable->recorder_out_.percentage = ui->spinBox_comparision_percentage->value();
    variable->delay_.alert = ui->spinBox_alert->value();
    variable->delay_.danger = ui->doubleSpinBox_danger->value();
    variable->delay_.active_100ms = ui->checkBox_100ms->isChecked();
    variable->alert_latching_ = ui->checkBox_alert_latching->isChecked();
    variable->danger_latching_ = ui->checkBox_danger_latching->isChecked();
    this->close();
}


void Radial::on_pushButton_set_default_clicked() {
}

